/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2011 SSAB Oxelosund AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 **/

#ifndef ge_attr_h
#define ge_attr_h

#include "glow.h"

/* ge_attr.h -- Ge attribute editor */

/*! \file ge_attr.h
    \brief Contains the Attr class. */
/*! \addtogroup Ge */
/*@{*/

class GeDyn;

typedef struct {
	void	*value;
	char	name[80];
	int	type;
	int	size;
	double	minlimit;
	double	maxlimit;
	int	noedit;
	int	multiline;
    	int	mask;
} attr_sItem;

class AttrNav;

//! Attribute editor.
/*! Displays grow attributes and dynamic attributes for an object or graph. */
class Attr {
  public:
    Attr(
      void			*a_parent_ctx,
      void 			*a_object,
      attr_sItem  		*itemlist,
      int			item_cnt);
    void 	*parent_ctx;
    char 	name[80];
    AttrNav	*attrnav;
    int		input_open;
    void 	*object;
    void	(*close_cb) (Attr *);
    void	(*redraw_cb) (Attr *);
    int		(*get_subgraph_info_cb)( void *, char *, attr_sItem  **, 
                          int *);
    int		(*get_dyn_info_cb)( void *, GeDyn *, attr_sItem  **, 
                          int *);
    int 	(*reconfigure_attr_cb)( void *, void *,
			    attr_sItem **, int *, void **);
    void	(*store_cb) (void *, void *);
    int		(*recall_cb) (void *, void *, int, GeDyn **);
    int		(*set_data_cb) (void *, void *, GeDyn *);
    int		(*get_plant_select_cb) (void *, char *, int);
    int 	(*get_current_colors_cb)( void *, glow_eDrawType *, glow_eDrawType *,
						  glow_eDrawType *);
    void	*client_data;
    int         recall_idx;
    GeDyn 	*original_data;

    virtual void message( char severity, const char *message) {}
    virtual void set_prompt( const char *prompt) {}
    virtual void change_value() {}
    virtual int reconfigure_attr() {return 1;}
    virtual void store();
    virtual void recall_next();
    virtual void recall_prev();
    static int get_plant_select_c( void *attr_ctx, char *value, int size);
    static int get_current_colors_c( void *attr_ctx, glow_eDrawType *fill_color, 
				      glow_eDrawType *border_color, 
				      glow_eDrawType *text_color);    
    static int get_subgraph_info_c( void *attr_ctx, char *name, 
				     attr_sItem **itemlist, int *item_cnt);
    static int get_dyn_info_c( void *attr_ctx, GeDyn *dyn, 
				attr_sItem **itemlist, int *item_cnt);
    static void change_value_c( void *attr);
    static int reconfigure_attr_c( void *attr);
    static void message( void *attr, char severity, const char *message);
    virtual ~Attr();
};


/*@}*/
#endif








